@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@vk.crocodile.org
 
-  $Id: ACL.cpp 1487 2009-09-23 17:00:48Z vadim $
+  $Id$
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -36,6 +36,14 @@ string ciscoACL::addLine(const std::string &s)
     return printLastLine();
 }
 
+string ciscoACL::quoteLine(const string &s)
+{
+    if (quote_remarks && s.find(' ') != string::npos)
+        return "\"" + s + "\"";
+    else
+        return s;
+}
+
 /*
  * Adds remark to access list. Checks and adds each remark only
  * once. We use rule labels for remarks
@@ -45,7 +53,8 @@ string ciscoACL::addRemark(const std::string &rl, const std::string &comment)
     string output;
     if (_last_rule_label != rl)
     {
-        acl.push_back(" remark " + rl);
+        acl.push_back(" remark " + quoteLine(rl));
+
         output += printLastLine();
         nlines++;
 
@@ -55,12 +64,12 @@ string ciscoACL::addRemark(const std::string &rl, const std::string &comment)
             c1 = 0;
             while ( (n = comment.find("\n", c1)) != string::npos )
             {
-                acl.push_back(" remark " + comment.substr(c1, n-c1));
+                acl.push_back(" remark " + quoteLine(comment.substr(c1, n-c1)));
                 output += printLastLine();
                 nlines++;
                 c1 = n + 1;
             }
-            acl.push_back(" remark " + comment.substr(c1));
+            acl.push_back(" remark " + quoteLine(comment.substr(c1)));
             output += printLastLine();
             nlines++;
         }
