@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@vk.crocodile.org
 
-  $Id: CompilerDriver_iosacl.cpp 2898 2010-05-12 15:08:27Z vadim $
+  $Id$
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -216,6 +216,13 @@ string CompilerDriver_iosacl::safetyNetInstall(Firewall *fw)
                                           (itype == "8021q"));
                     configlet.setVariable("management_interface_is_not_vlan",
                                           (itype != "8021q"));
+
+                    if (itype == "8021q")
+                        configlet.setVariable("management_interface_vlan_id",
+                                              ifopt->getInt("vlan_id"));
+                    else
+                        configlet.setVariable("management_interface_vlan_id", "");
+
                     break;
                 }
             }
