@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@vk.crocodile.org
 
-  $Id: Helper.cpp 2873 2010-05-02 20:42:26Z vadim $
+  $Id$
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -34,11 +34,17 @@
 #include <fwbuilder/Rule.h>
 #include "fwbuilder/Resources.h"
 #include <fwbuilder/IPv6.h>
+#include <fwbuilder/NetworkIPv6.h>
+#include <fwbuilder/Network.h>
+#include <fwbuilder/InetAddrMask.h>
 
 #include <assert.h>
 #include <limits.h>
 #include <iostream>
 
+#include <QObject>
+#include <QString>
+
 using namespace libfwbuilder;
 using namespace fwcompiler;
 using namespace std;
@@ -90,7 +96,7 @@ void Helper::expand_group_recursive(FWObject *o,list<FWObject*> &ol)
 
 int  Helper::findInterfaceByAddress(Address *obj)
 {
-    return findInterfaceByAddress( obj->getAddressPtr() );
+    return findInterfaceByAddress(obj->getAddressPtr(), obj->getNetmaskPtr());
 }
 
 /*
@@ -103,23 +109,88 @@ int  Helper::findInterfaceByAddress(Address *obj)
  * interfaces instead.
  */
 
-int  Helper::findInterfaceByAddress(const InetAddr *addr)
+int  Helper::findInterfaceByAddress(const InetAddr *addr,
+                                    const InetAddr *nm)
 {
     if (addr==NULL) return -1;
 
-    Firewall *fw=compiler->fw;
-    list<FWObject*> l2=fw->getByTypeDeep(Interface::TYPENAME);
+#if DEBUG_NETZONE_OPS
+    cerr << "Helper::findInterfaceByAddress";
+    cerr << " addr=" << addr->toString();
+    cerr << " nm=" << nm->toString();
+    cerr << endl;
+#endif
+
+    Firewall *fw = compiler->fw;
+    list<FWObject*> l2 = fw->getByTypeDeep(Interface::TYPENAME);
     for (list<FWObject*>::iterator i=l2.begin(); i!=l2.end(); ++i)
     {
-	Interface *iface=Interface::cast(*i);
+	Interface *iface = Interface::cast(*i);
         if (iface->isDedicatedFailover()) continue;
         if (iface->isUnprotected()) continue;
+
+#if DEBUG_NETZONE_OPS
+        cerr << "Helper::findInterfaceByAddress";
+        cerr << " intf=" << iface->getName();
+        cerr << endl;
+#endif
+
         FWObjectTypedChildIterator j =
             iface->findByType((addr->isV4())?IPv4::TYPENAME:IPv6::TYPENAME);
         for (; j!=j.end(); ++j)
         {
             const Address *i_addr = Address::constcast(*j);
-            if ( i_addr->belongs(*addr) ) return iface->getId();
+
+#if DEBUG_NETZONE_OPS
+            cerr << "Helper::findInterfaceByAddress";
+            cerr << " i_addr=" << i_addr->getName();
+            cerr << endl;
+            cerr << "    " << i_addr->getAddressPtr()->toString();
+            cerr << "    " << i_addr->getNetmaskPtr()->toString();
+            cerr << endl;
+#endif
+
+            if (nm != NULL)
+            {
+                InetAddrMask interface_subnet(*(i_addr->getAddressPtr()),
+                                              *(i_addr->getNetmaskPtr()));
+                InetAddrMask other_subnet(*addr, *nm);
+
+#if DEBUG_NETZONE_OPS
+                cerr << "Helper::findInterfaceByAddress";
+                cerr << " addr=" << other_subnet.toString();
+                cerr << " intf=" << iface->getName()
+                     << "  " << interface_subnet.toString();
+                cerr << endl;
+#endif
+
+                vector<InetAddrMask> ovr =
+                    libfwbuilder::getOverlap(interface_subnet, other_subnet);
+
+#if DEBUG_NETZONE_OPS
+                cerr << "Helper::findInterfaceByAddress";
+                cerr << " overlap:";
+                cerr << " ovr.size()=" << ovr.size();
+                if (ovr.size() > 0)
+                    cerr << " ovr.front()=" << ovr.front().toString();
+                cerr << endl;
+#endif
+                if (ovr.size()==0) continue;
+
+                // if interface_subnet is equal or wider than other_subnet,
+                // getOverlap() returns subnet object equal to other_subnet
+                // If other_subnet is wider, returned object is equal
+                // to interface_subnet. If they intersect but one does not fit
+                // completely in the other, returned object is not equal
+                // to either.
+                if (ovr.front() == other_subnet)
+                {
+                    return iface->getId();
+                }
+            } else
+            {
+                if ( i_addr->belongs(*addr) ) return iface->getId();
+            }
         }
     }
     return -1;
@@ -127,15 +198,24 @@ int  Helper::findInterfaceByAddress(const InetAddr *addr)
 
 int  Helper::findInterfaceByNetzone(Address *obj)
 {
-    return findInterfaceByNetzone(obj->getAddressPtr());
+    return findInterfaceByNetzone(obj->getAddressPtr(), obj->getNetmaskPtr());
 }
 
-int  Helper::findInterfaceByNetzone(const InetAddr *addr) throw(string)
+/**
+ * finds interface of the firewall associated with the netzone
+ * that object 'obj' belongs to.  Returns interface ID
+ *
+ */
+int  Helper::findInterfaceByNetzone(
+    const InetAddr *addr, const libfwbuilder::InetAddr *nm) throw(FWException)
 {
 #if DEBUG_NETZONE_OPS
-    cerr << "Helper::findInterfaceByNetzone "
-         << " matching to " << addr;
+    cerr << "Helper::findInterfaceByNetzone";
+    cerr << " matching to";
+    cerr << " addr=" << addr;
     if (addr) cerr << " " << addr->toString();
+    cerr << " nm=" << nm;
+    if (nm) cerr << " " << nm->toString();
     cerr << endl;
 #endif
 
@@ -159,7 +239,8 @@ int  Helper::findInterfaceByNetzone(const InetAddr *addr) throw(string)
             expand_group_recursive(netzone, nz);
 
 #if DEBUG_NETZONE_OPS
-            cerr << "netzone_id=" << netzone_id
+            cerr << "Helper::findInterfaceByNetzone";
+            cerr << "  netzone_id=" << netzone_id
                  << "  " << iface->getStr("network_zone")
                  << "  " << netzone->getName()
                  << endl;
@@ -170,6 +251,7 @@ int  Helper::findInterfaceByNetzone(const InetAddr *addr) throw(string)
                 if (Address::cast(*j) == NULL) continue;
 
 #if DEBUG_NETZONE_OPS
+                cerr << "Helper::findInterfaceByNetzone";
                 cerr << "    " << (*j)->getName()
                      << "  " << Address::cast(*j)->getAddressPtr()->toString()
                      << endl;
@@ -183,13 +265,51 @@ int  Helper::findInterfaceByNetzone(const InetAddr *addr) throw(string)
                         return iface->getId(); // id of the interface
                 } else
                 {
-                    if (Address::cast(*j)->belongs(*addr))
+                    const InetAddr *nz_addr = Address::cast(*j)->getAddressPtr();
+                    const InetAddr *nz_netm = Address::cast(*j)->getNetmaskPtr();
+                    if (nm != NULL && nz_netm != NULL)
                     {
-                        zones[iface->getId()] = *j;
+                        InetAddrMask nz_subnet(*nz_addr, *nz_netm);
+                        InetAddrMask other_subnet(*addr, *nm);
+                        vector<InetAddrMask> ovr =
+                            libfwbuilder::getOverlap(nz_subnet,
+                                                     other_subnet);
+#if DEBUG_NETZONE_OPS
+                        cerr << "Helper::findInterfaceByNetzone";
+                        cerr << " addr=" << other_subnet.toString();
+                        cerr << " nz=" << nz_subnet.toString();
+                        cerr << " overlap:";
+                        cerr << " ovr.size()=" << ovr.size();
+                        if (ovr.size() > 0)
+                            cerr << " ovr.front()=" << ovr.front().toString();
+                        cerr << endl;
+#endif
+                        if (ovr.size()==0) continue;
+                        // if nz_subnet is equal or wider than other_subnet,
+                        // getOverlap() returns subnet object equal to other_subnet
+                        // If other_subnet is wider, returned object is equal
+                        // to nz_subnet. If they intersect but one does not fit
+                        // completely in the other, returned object is not equal
+                        // to either.
+                        if (ovr.front() == other_subnet)
+                        {
+                            zones[iface->getId()] = *j;
+#if DEBUG_NETZONE_OPS
+                            cerr << "Helper::findInterfaceByNetzone";
+                            cerr << "    match" << endl;
+#endif
+                        }
+                    } else
+                    {
+                        if (Address::cast(*j)->belongs(*addr))
+                        {
+                            zones[iface->getId()] = *j;
 
 #if DEBUG_NETZONE_OPS
-                        cerr << "    match" << endl;
+                            cerr << "Helper::findInterfaceByNetzone";
+                            cerr << "    match" << endl;
 #endif
+                        }
                     }
                 }
             }
@@ -209,6 +329,7 @@ int  Helper::findInterfaceByNetzone(const InetAddr *addr) throw(string)
         unsigned long dim = calculateDimension(netzone);
 
 #if DEBUG_NETZONE_OPS
+        cerr << "Helper::findInterfaceByNetzone";
         cerr << "    netzone=" << netzone->getName()
              << "  dim=" << dim
              << "  res_dim=" << res_dim
@@ -222,15 +343,32 @@ int  Helper::findInterfaceByNetzone(const InetAddr *addr) throw(string)
         }
     }
 
+#if DEBUG_NETZONE_OPS
+    cerr << "Helper::findInterfaceByNetzone";
+    cerr << " Result after scanning network zones: " << res_id << endl;
+#endif
+
 /*
  * Subnets defined by addresses of interfaces are automatically part
  * of the corresponding network zones
  */
-    if (res_id == -1) res_id = findInterfaceByAddress( addr );
+    if (res_id == -1)
+        res_id = findInterfaceByAddress(addr, nm);
 
     if (res_id == -1)
-        throw(string("Can not find interface with network zone that includes "
-                     "address ") + string((addr)?addr->toString():"NULL"));
+    {
+        QString err = QObject::tr("Can not find interface with network zone "
+                                  "that includes address '%1%2'");
+        throw(FWException(err
+                          .arg((addr)?addr->toString().c_str():"NULL")
+                          .arg((nm)?QString("/%1").arg(nm->toString().c_str()):"")
+                          .toStdString()));
+    }
+
+#if DEBUG_NETZONE_OPS
+    cerr << "Helper::findInterfaceByNetzone";
+    cerr << " returning " << res_id << endl;
+#endif
 
     return res_id;
 }
@@ -252,7 +390,6 @@ list<int> Helper::getAllInterfaceIDs()
 
 list<int> Helper::findInterfaceByNetzoneOrAll(RuleElement *re)
 {
-    Firewall *fw = compiler->fw;
     list<int> intf_id_list;
     if (re->isAny())
     {
@@ -277,7 +414,7 @@ list<int> Helper::findInterfaceByNetzoneOrAll(RuleElement *re)
         {
             int intf_id = findInterfaceByNetzone(a);
             intf_id_list.push_back(intf_id);
-        } catch(string err)
+        } catch(FWException &ex)
         {
             // could not find interface with netzone to match address 'a'
             // will assign rule to all interfaces. Act as if all interfaces
@@ -290,7 +427,7 @@ list<int> Helper::findInterfaceByNetzoneOrAll(RuleElement *re)
                     compiler->fw->getStr("platform"), "network_zones");
 
             if (supports_network_zones)
-                compiler->warning(err);
+                compiler->warning(ex.toString());
 
             FWObjectTypedChildIterator i = compiler->fw->findByType(
                 Interface::TYPENAME);
