@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@vk.crocodile.org
 
-  $Id: PolicyCompiler_cisco.h 2403 2010-01-21 03:19:38Z vadim $
+  $Id$
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -69,6 +69,32 @@ protected:
          */
         virtual std::string debugPrintRule(libfwbuilder::Rule *rule);
 
+
+        virtual ciscoACL* createACLObject(const std::string &n,
+                                          libfwbuilder::Interface *intf,
+                                          const std::string &d="in",
+                                          bool _ip_list=false);
+
+        /* see #1690. After recent changes (aug 2010) in how we deal with
+         * network zones, we do not assume that interfaces have network
+         * zone "any" if they don't have network zone configured at all.
+         * To work around this, will set network zone to "any" on all
+         * interfaces. Note that this needs to be done only for IOS ACL and
+         * Procurve ACL but not PIX where network zone must be configured
+         * by the user.
+         */
+        virtual void setAllNetworkZonesToAny();
+
+        /*
+         * complementary operation: sets all interface's network zones
+         * to blank to make sure compiler operates with predictable
+         * configuration. This can be important if user switches from
+         * platform that requires network zones (PIX) to the one that
+         * does not support them, but compiler code uses the same
+         * classes.
+         */
+        virtual void setAllNetworkZonesToNone();
+       
 	/**
 	 * drops dynamic interface from the rule in the following
 	 * cases because its address is unknown and we can not build
