@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@vk.crocodile.org
 
-  $Id: PolicyCompiler_cisco_acls.cpp 2884 2010-05-11 02:59:53Z vadim $
+  $Id$
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -241,7 +241,7 @@ bool PolicyCompiler_cisco::setInterfaceAndDirectionIfInterfaceSet::processNext()
 
 bool PolicyCompiler_cisco::pickACL::processNext()
 {
-    PolicyCompiler_cisco *cisco_comp=dynamic_cast<PolicyCompiler_cisco*>(
+    PolicyCompiler_cisco *cisco_comp = dynamic_cast<PolicyCompiler_cisco*>(
         compiler);
     PolicyRule *rule=getNext(); if (rule==NULL) return false;
     
@@ -318,7 +318,7 @@ bool PolicyCompiler_cisco::pickACL::processNext()
 
     rule->setStr("acl",acl_name);
 
-    ciscoACL *acl = new ciscoACL(acl_name, rule_iface, dir, using_named_acl);
+    ciscoACL *acl = cisco_comp->createACLObject(acl_name, rule_iface, dir, using_named_acl);
     cisco_comp->acls[acl_name] = acl;
 
     acl->setWorkName(acl_name);
