@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@vk.crocodile.org
 
-  $Id: CompilerDriver.cpp 3082 2010-07-14 23:22:49Z vadim $
+  $Id$
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -146,6 +146,7 @@ bool CompilerDriver::configure(const QStringList &args)
         }
         if (arg == "-d")
         {
+            // TODO: deal with UTF-8 in directory name
             idx++;
             wdir = string(args.at(idx).toLatin1().constData());
             continue;
@@ -252,6 +253,27 @@ void CompilerDriver::chDir()
     }
 }
 
+QString CompilerDriver::getAbsOutputFileName(const QString &output_file_name)
+{
+    QFileInfo finfo(output_file_name);
+    if (finfo.isRelative())
+    {
+        // if fw_file_name is relative, it is relative to the
+        // directory the program started in, or if wdir was defined
+        // via "-d" command line switch, then it is relative to that.
+        if (wdir.empty())
+        {
+            QFileInfo new_finfo(start_current_dir, output_file_name);
+            return new_finfo.absoluteFilePath();
+        } else
+        {
+            QFileInfo new_finfo(QDir(wdir.c_str()), output_file_name);
+            return new_finfo.absoluteFilePath();
+        }
+    }
+    return output_file_name;
+}
+
 void CompilerDriver::commonChecks(Firewall *fw)
 {
     if (Cluster::isA(fw))
@@ -637,28 +659,56 @@ void CompilerDriver::findImportedRuleSets(Firewall *fw,
     int cluster_id = fw->getInt("parent_cluster_id");
 
     list<FWObject*> imported_policies;
-    for (list<FWObject*>::iterator i=all_policies.begin();
-         i!=all_policies.end(); ++i)
+    for (list<FWObject*>::iterator i=all_policies.begin(); i!=all_policies.end(); ++i)
     {
-        for (list<FWObject*>::iterator r=(*i)->begin(); r!=(*i)->end(); ++r)
+        RuleSet *ruleset = RuleSet::cast(*i);
+        if (ruleset == NULL) continue; // should not happen
+
+        for (list<FWObject*>::iterator r=ruleset->begin(); r!=ruleset->end(); ++r)
         {
             Rule *rule = Rule::cast(*r);
             if (rule == NULL) continue; // skip RuleSetOptions object
-            RuleSet *ruleset = rule->getBranch();
 
-            if (ruleset!=NULL)
+            RuleSet *branch_ruleset = rule->getBranch();
+            if (branch_ruleset!=NULL)
             {
-                if (ruleset->isChildOf(fw)) continue;
+                map<FWObject*, int> referenced_branch_rulesets;
 
-                // Additional check: the rule set may be child of a
-                // cluster this firewall is member of. If it is, it
-                // has been taken care of in CompilerDriver::mergeRuleSets()
-                FWObject *ruleset_parent = ruleset->getParent();
-                if (cluster_member && Cluster::isA(ruleset_parent) &&
-                    ruleset_parent->getId() == cluster_id) continue;
+                _findImportedRuleSetsRecursively(
+                    fw, branch_ruleset, referenced_branch_rulesets);
 
-                ruleset->setTop(false);
-                imported_policies.push_back(ruleset);
+                map<FWObject*, int>::iterator it;
+                for (it=referenced_branch_rulesets.begin();
+                     it!=referenced_branch_rulesets.end(); ++it)
+                {
+                    RuleSet *branch_ruleset = RuleSet::cast(it->first);
+                    int counter = it->second;
+
+                    if (counter > 1)
+                    {
+                        QString err("Rule set %1 of firewall %2 has branching rule that loops back to it");
+                        warning(ruleset->getParent(), ruleset, rule,
+                                err.arg(ruleset->getName().c_str())
+                                .arg(fw->getName().c_str()).toStdString());
+                    }
+
+                    if (branch_ruleset->isChildOf(fw)) continue;
+
+                    list<FWObject*>::iterator it = std::find(imported_policies.begin(),
+                                                             imported_policies.end(),
+                                                             branch_ruleset);
+                    if (it != imported_policies.end()) continue;
+
+                    // Additional check: the rule set may be child of a
+                    // cluster this firewall is member of. If it is, it
+                    // has been taken care of in CompilerDriver::mergeRuleSets()
+                    FWObject *ruleset_parent = branch_ruleset->getParent();
+                    if (cluster_member && Cluster::isA(ruleset_parent) &&
+                        ruleset_parent->getId() == cluster_id) continue;
+
+                    branch_ruleset->setTop(false);
+                    imported_policies.push_back(branch_ruleset);
+                }
             }
         }
     }
@@ -667,6 +717,27 @@ void CompilerDriver::findImportedRuleSets(Firewall *fw,
                             imported_policies.begin(), imported_policies.end());
 }
 
+void CompilerDriver::_findImportedRuleSetsRecursively(
+    Firewall *fw, RuleSet *branch_ruleset, map<FWObject*, int> &branch_rulesets)
+{
+    int c = branch_rulesets[branch_ruleset];
+    branch_rulesets[branch_ruleset] = ++c;
+    if (c > 1) return;  // we have seen this one already
+
+    for (list<FWObject*>::iterator r=branch_ruleset->begin(); r!=branch_ruleset->end(); ++r)
+    {
+        Rule *rule = Rule::cast(*r);
+        if (rule == NULL) continue; // skip RuleSetOptions object
+
+        RuleSet *next_branch_ruleset = rule->getBranch();
+        if (next_branch_ruleset!=NULL)
+        {
+            _findImportedRuleSetsRecursively(fw, next_branch_ruleset, branch_rulesets);
+        }
+    }
+}
+
+
 QString CompilerDriver::run(const std::string&, const std::string&, const std::string&)
 {
     return "";
