@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@vk.crocodile.org
 
-  $Id: CompilerDriver.h 3079 2010-07-14 17:31:28Z a2k $
+  $Id$
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -134,6 +134,13 @@ protected:
                                               Configlet *top_comment,
                                               const QString &comment_char);
 
+        void _findImportedRuleSetsRecursively(libfwbuilder::Firewall *fw,
+                                              libfwbuilder::RuleSet *ruleset,
+                                              std::map<libfwbuilder::FWObject*, int> &branch_rulesets);
+        
+
+        QString getAbsOutputFileName(const QString &output_file_name);
+        
 public:
 
         CompilerDriver(libfwbuilder::FWObjectDatabase *db);
