@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: interfaceProperties.cpp 2958 2010-06-07 22:40:54Z vadim $
+  $Id$
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -65,7 +65,7 @@ bool interfaceProperties::looksLikeVlanInterface(const QString &int_name)
 
 // simple name validation: does not allow space and "-"
 // However some platform permit space (procurve).
-bool interfaceProperties::basicValidateInterfaceName(Interface *intf,
+bool interfaceProperties::basicValidateInterfaceName(Interface *,
                                                      const QString &obj_name,
                                                      QString &err)
 {
