@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: main.cpp 2968 2010-06-08 20:35:48Z vadim $
+  $Id$
 
 
   This program is free software which we release under the GNU General Public
@@ -112,6 +112,8 @@ int main( int argc, char *argv[] )
     fwbdebug = 0;
     safemode = false;
 
+    bool force_first_time_run_flag = false;
+
     ssh_wrapper(argc, argv);
 
     // can not use "-p" for command line printing because
@@ -119,7 +121,7 @@ int main( int argc, char *argv[] )
     // started via Finder.
 
     int c;
-    while ((c = getopt (argc , argv , "hvf:o:P:dxgr")) != EOF )
+    while ((c = getopt (argc , argv , "1hvf:o:P:dxgr")) != EOF )
 	switch (c) {
 	case 'h':
 	    usage();
@@ -148,13 +150,18 @@ int main( int argc, char *argv[] )
         case 'P':
             cli_print = true ;
             cli_print_fwname = optarg;
+            break;
+
+        case '1':
+            force_first_time_run_flag = true;
+            break;
 	}
 
     if ( (argc-1)==optind)
         filename = strdup( argv[optind++] );
 
     //QApplication::setDesktopSettingsAware(desktopaware);
-
+ 
     Q_INIT_RESOURCE(MainRes);
 
     app = new FWBApplication( argc, argv );
@@ -170,7 +177,7 @@ int main( int argc, char *argv[] )
 
     if (fwbdebug) qDebug("Reading settings ...");
     st = new FWBSettings();
-    st->init();
+    st->init(force_first_time_run_flag);
     if (fwbdebug) qDebug("done");
 
     wfl = new UserWorkflow();
@@ -214,7 +221,7 @@ int main( int argc, char *argv[] )
                                         print_output_file_name);
         return 0;
     }
-
+    
 
     if (fwbdebug) qDebug("creating widgets ...");
 
