@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@vk.crocodile.org
 
-  $Id: CompilerDriver_ipt_policy.cpp 2868 2010-05-01 22:37:47Z vadim $
+  $Id$
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -219,12 +219,17 @@ bool CompilerDriver_ipt::processPolicyRuleSet(
         if (flush_and_set_default_policy)
             tmp << policy_compiler->flushAndSetDefaultPolicy();
 
+        /*
+         * commented out for #1707
+         *
+
         if (!prolog_done && prolog_place == "after_flush" &&
             !fw->getOptionsObject()->getBool("use_iptables_restore"))
         {
             tmp << "prolog_commands" << endl;
             prolog_done = true;
         }
+        */
 
         tmp << policy_compiler->printAutomaticRules();
 
