@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@vk.crocodile.org
 
-  $Id: CompilerDriver_ipt_run.cpp 3079 2010-07-14 17:31:28Z a2k $
+  $Id$
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -201,6 +201,7 @@ QString CompilerDriver_ipt::run(const std::string &cluster_id,
         list<FWObject*> all_nat = fw->getByType(NAT::TYPENAME);
 
         int routing_rules_count = 0;
+        bool have_ipv4 = false;
         bool have_ipv6 = false;
 
         // track chains in each table separately. Can we have the same
@@ -365,6 +366,7 @@ QString CompilerDriver_ipt::run(const std::string &cluster_id,
                     generated_script += "\n\n";
                 } else
                 {
+                    have_ipv4 = true;
                     generated_script += "\n\n";
                     generated_script += "# ================ IPv4\n";
                     generated_script += "\n\n";
@@ -427,7 +429,6 @@ QString CompilerDriver_ipt::run(const std::string &cluster_id,
  * These store generated configuration internally, extract it later using
  * OSConfiguration::getGeneratedFiles();
  */
-        oscnf->printChecksForRunTimeMultiAddress();
         oscnf->processFirewallOptions();
 
 /*
@@ -488,11 +489,14 @@ QString CompilerDriver_ipt::run(const std::string &cluster_id,
                                     oscnf->printPathForAllTools(os_variant).c_str());
         script_skeleton.setVariable("shell_functions",
                                     oscnf->printShellFunctions(have_ipv6).c_str());
+        script_skeleton.setVariable("run_time_address_tables",
+                                    oscnf->printRunTimeAddressTablesCode().c_str());
+        script_skeleton.setVariable("using_ipset", oscnf->usingIpSetModule());
+
         if (supports_prolog_epilog)
         {
             //script_skeleton.setVariable("prolog_epilog",
             //                         oscnf->printPrologEpilogFunctions().c_str());
-
             script_skeleton.setVariable(
                 "prolog_script", 
                 fw->getOptionsObject()->getStr("prolog_script").c_str());
@@ -584,7 +588,8 @@ QString CompilerDriver_ipt::run(const std::string &cluster_id,
 
         script_skeleton.setVariable("load_modules",
                                     oscnf->generateCodeForProtocolHandlers().c_str());
-        script_skeleton.setVariable("load_modules_with_nat", (have_nat)?"\"nat\"":"");
+        script_skeleton.setVariable("load_modules_with_nat", (have_nat)?"nat":"");
+        script_skeleton.setVariable("load_modules_with_ipv6", (have_ipv6)?"ipv6":"");
 
         script_skeleton.setVariable("ip_forward_commands",
                                     oscnf->printIPForwardingCommands().c_str());
@@ -627,28 +632,41 @@ QString CompilerDriver_ipt::run(const std::string &cluster_id,
                                     ! fw->getOptionsObject()->getBool("use_iptables_restore"));
 
         script_buffer = "";
-        script << "  reset_iptables_v4" << endl;
+        if (have_ipv4) script << "  reset_iptables_v4" << endl;
         if (have_ipv6) script << "  reset_iptables_v6" << endl;
         script_skeleton.setVariable("reset_all", script_buffer);
 
         script_buffer = "";
 
-        Configlet stop_action(fw, "linux24", "stop_action");
-        stop_action.collapseEmptyStrings(true);
+        Configlet block_action(fw, "linux24", "block_action");
+        block_action.collapseEmptyStrings(true);
 
+        // the name of the option is historical (including the typo)
         if (fw->getOptionsObject()->getBool("add_mgmt_ssh_rule_when_stoped"))
         {
-            std::auto_ptr<PolicyCompiler_ipt> policy_compiler = createPolicyCompiler(
-                fw, false, NULL,  NULL);
+            std::auto_ptr<PolicyCompiler_ipt> policy_compiler =
+                createPolicyCompiler(fw, false, NULL,  NULL);
             PolicyCompiler_ipt::PrintRule* print_rule =
                 policy_compiler->createPrintRuleProcessor();
             print_rule->setContext(policy_compiler.get());
-            print_rule->_printBackupSSHAccessRules(&stop_action);
+            print_rule->_printBackupSSHAccessRules(&block_action);
         } else
-            stop_action.setVariable("mgmt_access", 0);
+        {
+            block_action.setVariable("mgmt_access", 0);
+        }
+
+        script_skeleton.setVariable("block_action", block_action.expand());
+
+
+        Configlet stop_action(fw, "linux24", "stop_action");
+        stop_action.collapseEmptyStrings(true);
+        stop_action.setVariable("have_ipv4", have_ipv4);
+        stop_action.setVariable("have_ipv6", have_ipv6);
 
         script_skeleton.setVariable("stop_action", stop_action.expand());
 
+
+
         Configlet status_action(fw, "linux24", "status_action");
         status_action.collapseEmptyStrings(true);
         script_skeleton.setVariable("status_action", status_action.expand());
@@ -698,17 +716,7 @@ QString CompilerDriver_ipt::run(const std::string &cluster_id,
         script_skeleton.setVariable("errors_and_warnings",
                                     prepend("# ", all_errors.join("\n")));
 
-        QFileInfo finfo(fw_file_name);
-        if (finfo.isRelative())
-        {
-            // if fw_file_name is relative, it is relative to the
-            // directory the program started in, which can be
-            // different from wdir and different from the current dir
-            // at this point because we do chdir to the directory
-            // defined by the -d command line option
-            QFileInfo new_finfo(start_current_dir, fw_file_name);
-            fw_file_name = new_finfo.absoluteFilePath();
-        }
+        fw_file_name = getAbsOutputFileName(fw_file_name);
 
         info("Output file name: " + fw_file_name.toStdString());
 
