@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@vk.crocodile.org
 
-  $Id: NATCompiler_PrintRule.cpp 2449 2010-01-27 01:06:12Z vadim $
+  $Id$
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -60,6 +60,7 @@
 
 #include <assert.h>
 
+#include <QString>
 #include <QStringList>
 
 using namespace libfwbuilder;
@@ -501,6 +502,20 @@ string NATCompiler_ipt::PrintRule::_printDstService(RuleElementOSrv  *rel)
     return ostr.str();
 }
 
+string NATCompiler_ipt::PrintRule::_printIpSetMatch(Address  *o, RuleElement *rel)
+{
+    NATCompiler_ipt *ipt_comp=dynamic_cast<NATCompiler_ipt*>(compiler);
+    string set_name =
+        dynamic_cast<OSConfigurator_linux24*>(ipt_comp->osconfigurator)->normalizeSetName(o->getName());
+    string suffix = "dst";
+    if (RuleElementOSrc::isA(rel)) suffix = "src";
+    if (RuleElementODst::isA(rel)) suffix = "dst";
+    string set_match = "--set " + set_name + " " + suffix;
+    ostringstream ostr;
+    ostr << "-m set " << _printSingleOptionWithNegation("", rel, set_match);
+    return ostr.str();
+}
+
 // Note print_mask is true by default, print_range is false by default.
 string NATCompiler_ipt::PrintRule::_printAddr(Address  *o,
                                               bool ,
@@ -659,9 +674,17 @@ bool NATCompiler_ipt::PrintRule::processNext()
         const InetAddr *osrc_addr = osrc->getAddressPtr();
         if (osrc_addr==NULL || !osrc_addr->isAny())
         {
-            string osrc_out = _printAddr(osrc);
-            if (!osrc_out.empty())
-                cmdout << _printSingleOptionWithNegation(" -s", osrcrel, osrc_out);
+            MultiAddressRunTime *atrt = MultiAddressRunTime::cast(osrc);
+            if (atrt!=NULL && atrt->getSubstitutionTypeName()==AddressTable::TYPENAME &&
+                ipt_comp->using_ipset)
+            {
+                cmdout << _printIpSetMatch(osrc, osrcrel);
+            } else
+            {
+                string osrc_out = _printAddr(osrc);
+                if (!osrc_out.empty())
+                    cmdout << _printSingleOptionWithNegation(" -s", osrcrel, osrc_out);
+            }
         }
 
 //	cmdout << " -s ";
@@ -676,7 +699,13 @@ bool NATCompiler_ipt::PrintRule::processNext()
 
     if (!odst->isAny())
     {
-        cmdout << _printSingleOptionWithNegation(" -d", odstrel, _printAddr(odst));
+        MultiAddressRunTime *atrt = MultiAddressRunTime::cast(odst);
+        if (atrt!=NULL && atrt->getSubstitutionTypeName()==AddressTable::TYPENAME &&
+            ipt_comp->using_ipset)
+        {
+            cmdout << _printIpSetMatch(odst, odstrel);
+        } else
+            cmdout << _printSingleOptionWithNegation(" -d", odstrel, _printAddr(odst));
     }
 
     cmdout << " ";
@@ -696,9 +725,21 @@ bool NATCompiler_ipt::PrintRule::processNext()
 	    cmdout << "--to-source ";
             // if TSrc is "any" and this is SNAT rule, then this rule only
             // translates source port. Skip address part.
-            if (!tsrc->isAny()) cmdout << _printAddr(tsrc, false, true);
+            // Note for #1693: _printAddr() returns string that ends with a space,
+            // but there should be no space if the address is followed by port
+            // spec.
+            QString addr_part;
+            if (!tsrc->isAny())
+            {
+                addr_part = _printAddr(tsrc, false, true).c_str();
+            }
 	    string ports = _printSNATPorts(tsrv);
-	    if (!ports.empty()) cmdout << ":" << ports;
+	    if (!ports.empty())
+            {
+                cmdout << addr_part.trimmed().toStdString()
+                       << ":" << ports;
+            } else
+                cmdout << addr_part.toStdString();
 
             if (ropt->getBool("ipt_nat_random")) cmdout << " --random";
             if (XMLTools::version_compare(version, "1.4.3")>=0)
@@ -717,9 +758,19 @@ bool NATCompiler_ipt::PrintRule::processNext()
 	    cmdout << "--to-destination ";
             // if TDst is "any" and this is DNAT rule, then this rule only
             // translates source port. Skip address part.
-            if (!tdst->isAny()) cmdout << _printAddr(tdst, false, true);
+            QString addr_part;
+            if (!tdst->isAny())
+                addr_part = _printAddr(tdst, false, true).c_str();
+
 	    string ports = _printDNATPorts(tsrv);
-	    if (!ports.empty()) cmdout << ":" << ports;
+	    if (!ports.empty())
+            {
+                cmdout << addr_part.trimmed().toStdString()
+                       << ":" << ports;
+            } else
+            {
+                cmdout << addr_part.toStdString();
+            }
 
             if (ropt->getBool("ipt_nat_random")) cmdout << " --random";
             if (XMLTools::version_compare(version, "1.4.3")>=0)
