@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@vk.crocodile.org
 
-  $Id: NATCompiler_ipt.cpp 3022 2010-06-25 03:38:24Z vadim $
+  $Id$
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -112,12 +112,23 @@ const std::list<std::string>& NATCompiler_ipt::getStandardChains()
 
 string NATCompiler_ipt::myPlatformName() { return "iptables"; }
 
+/*
+ * this function generates acceptable shell variable name from
+ * interface name. Note that
+ * OSConfigurator_linux24::getInterfaceVarName() and
+ * PolicyCompiler_ipt::getInterfaceVarName() do the same thing and
+ * these functions should be identical.
+ *
+ *  TODO: really need to have one function for this instead of three in
+ * three different classes.
+ */
 string NATCompiler_ipt::getInterfaceVarName(FWObject *iface, bool v6)
 {
     ostringstream  ostr;
     string iname=iface->getName();
     string::size_type p1;
     while ( (p1=iname.find("."))!=string::npos) iname=iname.replace(p1,1,"_");
+    while ( (p1=iname.find("-"))!=string::npos) iname=iname.replace(p1,1,"_");
     ostr << "i_" << iname;
     if (v6) ostr << "_v6";
     return ostr.str();
@@ -214,6 +225,10 @@ int NATCompiler_ipt::prolog()
 	}
     }
 
+    string version = fw->getStr("version");
+    using_ipset = (XMLTools::version_compare(version, "1.4.1.1") >= 0 &&
+                          fwopt->getBool("use_m_set"));
+
     return n;
 }
 
@@ -665,7 +680,7 @@ bool NATCompiler_ipt::VerifyRules2::processNext()
             return true;
         }
 
-        if (!tsrv->isAny() && s1->getProtocolNumber()!=s2->getProtocolNumber())
+        if (!tsrv->isAny() && s1->getProtocolName()!=s2->getProtocolName())
         {
             compiler->abort(
                 rule, 
@@ -732,12 +747,26 @@ bool NATCompiler_ipt::portTranslationRules::processNext()
 
 bool NATCompiler_ipt::specialCaseWithRedirect::processNext()
 {
-    NATRule *rule=getNext(); if (rule==NULL) return false;
+    NATRule *rule = getNext(); if (rule==NULL) return false;
 
-    Address  *tdst=compiler->getFirstTDst(rule);
+    Address *tdst = compiler->getFirstTDst(rule);
 
-/* we consider rule redirect only if TDst is a firewall object */
-    if (rule->getRuleType() == NATRule::DNAT && tdst->getId()==compiler->fw->getId()) 
+/* we consider rule redirect only if TDst is a firewall object
+ *
+ */
+    int fw_id = compiler->fw->getId();
+    int cluster_id = -1;
+    bool cluster_member = compiler->fw->getOptionsObject()->getBool("cluster_member");
+    Cluster *cluster = NULL;
+    if (cluster_member)
+    {
+        cluster = Cluster::cast(
+            compiler->dbcopy->findInIndex(compiler->fw->getInt("parent_cluster_id")));
+        cluster_id = cluster->getId();
+    }
+
+    if (rule->getRuleType() == NATRule::DNAT && 
+        (tdst->getId() == fw_id || tdst->getId() == cluster_id))
         rule->setRuleType(NATRule::Redirect);
 
     tmp_queue.push_back(rule);
@@ -824,7 +853,6 @@ bool NATCompiler_ipt::fillTranslatedSrv::processNext()
 
 bool NATCompiler_ipt::addVirtualAddress::processNext()
 {
-    FWOptions* options=compiler->fw->getOptionsObject();
     NATRule *rule=getNext(); if (rule==NULL) return false;
 
     tmp_queue.push_back(rule);
@@ -1913,8 +1941,8 @@ bool NATCompiler_ipt::splitNONATRule::processNext()
 {
     NATRule *rule=getNext(); if (rule==NULL) return false;
 
-    if ( rule->getStr("ipt_chain").empty() && rule->getRuleType()==NATRule::NONAT) {
-
+    if ( rule->getStr("ipt_chain").empty() && rule->getRuleType()==NATRule::NONAT)
+    {
         Address *osrc=compiler->getFirstOSrc(rule);
         bool osrcfw= compiler->complexMatch(osrc,compiler->fw);
 /*
@@ -2003,29 +2031,31 @@ bool NATCompiler_ipt::splitNATBranchRule::processNext()
                         }
 
                     }
+                    return true;
                 }
-            } else
-            {
-                compiler->warning(rule,
-                                  "NAT branching rule does not have information"
-                                  " about targets used in the branch ruleset"
-                                  " to choose proper chain in the nat table."
-                                  " Will split the rule and place it in both"
-                                  " PREROUTNING and POSTROUTING");
-                NATRule *r = compiler->dbcopy->createNATRule();
-                compiler->temp_ruleset->add(r);
-                r->duplicate(rule);
-                r->setStr("ipt_chain", "POSTROUTING");
-                r->setStr("ipt_target", branch_name);
-                tmp_queue.push_back(r);
-
-                r = compiler->dbcopy->createNATRule();
-                compiler->temp_ruleset->add(r);
-                r->duplicate(rule);
-                r->setStr("ipt_chain", "PREROUTING");
-                r->setStr("ipt_target", branch_name);
-                tmp_queue.push_back(r);
             }
+
+            compiler->warning(rule,
+                              "NAT branching rule does not have information"
+                              " about targets used in the branch ruleset"
+                              " to choose proper chain in the nat table."
+                              " Will split the rule and place it in both"
+                              " PREROUTNING and POSTROUTING");
+            NATRule *r = compiler->dbcopy->createNATRule();
+            compiler->temp_ruleset->add(r);
+            r->duplicate(rule);
+            r->setStr("ipt_chain", "POSTROUTING");
+            r->setStr("ipt_target", branch_name);
+            tmp_queue.push_back(r);
+
+            r = compiler->dbcopy->createNATRule();
+            compiler->temp_ruleset->add(r);
+            r->duplicate(rule);
+            r->setStr("ipt_chain", "PREROUTING");
+            r->setStr("ipt_target", branch_name);
+            tmp_queue.push_back(r);
+
+            return true;
         }
         else 
         {
@@ -2059,7 +2089,7 @@ bool NATCompiler_ipt::localNATRule::processNext()
     {
     case NATRule::DNAT:
     case NATRule::DNetnat:
-
+    case NATRule::Redirect:
 /* it should not be necessary to do anything if rule type is NONAT
  * since splitNONATRule takes care of NONAT rules
  *
@@ -2280,21 +2310,20 @@ bool NATCompiler_ipt::AssignInterface::processNext()
              * Include interfaces that have no addresses in the list
              * for backwards compatibility.
              */
-            FWObjectTypedChildIterator ipv4_addresses = iface->findByType(IPv4::TYPENAME);
-            FWObjectTypedChildIterator ipv6_addresses = iface->findByType(IPv6::TYPENAME);
+            FWObjectTypedChildIterator ipv4_addresses =
+                iface->findByType(IPv4::TYPENAME);
+            FWObjectTypedChildIterator ipv6_addresses =
+                iface->findByType(IPv6::TYPENAME);
 
             if ((ipt_comp->ipv6 && ipv6_addresses != ipv6_addresses.end()) ||
                 (!ipt_comp->ipv6 && ipv4_addresses != ipv4_addresses.end()) ||
-                ipv4_addresses == ipv4_addresses.end() && ipv6_addresses == ipv6_addresses.end())
+                (ipv4_addresses == ipv4_addresses.end() && ipv6_addresses == ipv6_addresses.end()))
             {
                 /* 
                  * if interface name ends with '*', this is wildcard
                  * interface. Just replace '*' with '+'. If interace
                  * name does not end with '*', replace numeric
-                 * interface index with '+'. Either way, cptr points
-                 * at the first caracter after the 'family' name of
-                 * the interface (is there a better term?) which will
-                 * be either a digit or '*'.
+                 * interface index with '+'. 
                  */
 
                 QString iname = QString(iface->getName().c_str());
@@ -2484,7 +2513,8 @@ bool NATCompiler_ipt::processMultiAddressObjectsInRE::processNext()
     if (re->size()==1) 
     {
         FWObject *o = re->front();
-        if (FWReference::cast(o)!=NULL) o=FWReference::cast(o)->getPointer();
+        if (FWReference::cast(o)!=NULL) o = FWReference::cast(o)->getPointer();
+
         MultiAddressRunTime *atrt = MultiAddressRunTime::cast(o);
         if (atrt!=NULL)
         {
@@ -2615,11 +2645,19 @@ void NATCompiler_ipt::compile()
                  " swap MultiAddress -> MultiAddressRunTime in OSrc") );
         add( new swapMultiAddressObjectsInODst(
                  " swap MultiAddress -> MultiAddressRunTime in ODst") );
+        add( new swapMultiAddressObjectsInTSrc(
+                 " swap MultiAddress -> MultiAddressRunTime in TSrc") );
+        add( new swapMultiAddressObjectsInTDst(
+                 " swap MultiAddress -> MultiAddressRunTime in TDst") );
 
         add( new processMultiAddressObjectsInOSrc(
                  "process MultiAddress objects in OSrc") );
         add( new processMultiAddressObjectsInODst(
                  "process MultiAddress objects in ODst") );
+        add( new processMultiAddressObjectsInTSrc(
+                 "process MultiAddress objects in TSrc") );
+        add( new processMultiAddressObjectsInTDst(
+                 "process MultiAddress objects in TDst") );
 
         add( new doOSrvNegation( "process negation in OSrv" ));
 
