@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@vk.crocodile.org
 
-  $Id: NATCompiler_ipt.h 2800 2010-04-09 02:29:27Z vadim $
+  $Id$
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -60,7 +60,8 @@ namespace fwcompiler {
         class PrintRule;
 
         NATCompiler_ipt::PrintRule *printRule;
-        bool   have_dynamic_interfaces;
+        bool have_dynamic_interfaces;
+        bool using_ipset;
         std::map<std::string, int> chain_usage_counter;
 
         // use minus_n_commands map to track creation of chains.
@@ -466,16 +467,33 @@ namespace fwcompiler {
         {
             public:
             processMultiAddressObjectsInOSrc(const std::string &n) :
-                processMultiAddressObjectsInRE(n,libfwbuilder::RuleElementOSrc::TYPENAME) {}
+                processMultiAddressObjectsInRE(
+                    n, libfwbuilder::RuleElementOSrc::TYPENAME) {}
         };
 
         class processMultiAddressObjectsInODst : public processMultiAddressObjectsInRE
         {
             public:
             processMultiAddressObjectsInODst(const std::string &n) :
-                processMultiAddressObjectsInRE(n,libfwbuilder::RuleElementODst::TYPENAME) {}
+                processMultiAddressObjectsInRE(
+                    n, libfwbuilder::RuleElementODst::TYPENAME) {}
         };
 
+        class processMultiAddressObjectsInTSrc : public processMultiAddressObjectsInRE
+        {
+            public:
+            processMultiAddressObjectsInTSrc(const std::string &n) :
+                processMultiAddressObjectsInRE(
+                    n, libfwbuilder::RuleElementTSrc::TYPENAME) {}
+        };
+
+        class processMultiAddressObjectsInTDst : public processMultiAddressObjectsInRE
+        {
+            public:
+            processMultiAddressObjectsInTDst(const std::string &n) :
+                processMultiAddressObjectsInRE(
+                    n, libfwbuilder::RuleElementTDst::TYPENAME) {}
+        };
 
 	/**
 	 * count how many times each user-defined chain we've created is
@@ -535,6 +553,8 @@ namespace fwcompiler {
             virtual std::string _printAddr(libfwbuilder::Address  *o,
                                            bool print_mask=true,
                                            bool print_range=false);
+            virtual std::string _printIpSetMatch(
+                libfwbuilder::Address *o, libfwbuilder::RuleElement *rel);
             virtual std::string _printChainDirectionAndInterface(libfwbuilder::NATRule *r);
             virtual std::string _printSingleObjectNegation(libfwbuilder::RuleElement *rel);
 
