@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@vk.crocodile.org
 
-  $Id: OSConfigurator_linux24.cpp 2751 2010-03-23 21:04:27Z vadim $
+  $Id$
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -76,6 +76,11 @@ OSConfigurator_linux24::OSConfigurator_linux24(FWObjectDatabase *_db,
     OSConfigurator(_db, fw, ipv6_policy) , os_data(fw->getStr("host_OS"))
 {
     command_wrappers = new Configlet(fw, "linux24", "run_time_wrappers");
+
+    FWOptions* fwopt = fw->getOptionsObject();
+    string version = fw->getStr("version");
+    using_ipset = (XMLTools::version_compare(version, "1.4.1.1") >= 0 &&
+                          fwopt->getBool("use_m_set"));
 }
 
 OSConfigurator_linux24::~OSConfigurator_linux24()
@@ -83,6 +88,15 @@ OSConfigurator_linux24::~OSConfigurator_linux24()
     delete command_wrappers;
 }
 
+/*
+ * this function generates acceptable shell variable name from
+ * interface name. Note that PolicyCompiler_ipt::getInterfaceVarName()
+ * and NATCompiler_ipt::getInterfaceVarName do the same thing and
+ * these functions should be identical.
+ *
+ * TODO: really need to have one function for this instead of three in
+ * three different classes.
+ */
 string OSConfigurator_linux24::getInterfaceVarName(FWObject *iface, bool v6)
 {
     ostringstream  ostr;
@@ -90,6 +104,8 @@ string OSConfigurator_linux24::getInterfaceVarName(FWObject *iface, bool v6)
     string::size_type p1;
     while ( (p1=iname.find("."))!=string::npos)
         iname=iname.replace(p1,1,"_");
+    while ( (p1=iname.find("-"))!=string::npos)
+        iname=iname.replace(p1,1,"_");
     ostr << "i_" << iname;
     if (v6) ostr << "_v6";
     return ostr.str();
@@ -265,22 +281,17 @@ void OSConfigurator_linux24::addVirtualAddressForNAT(const Address *addr)
     }
 }
 
-void OSConfigurator_linux24::registerMultiAddressObject(MultiAddressRunTime *at)
+string OSConfigurator_linux24::normalizeSetName(const string &txt)
 {
-    address_table_objects[at->getName()] = at->getSourceName();
+    QString table_name = txt.c_str();
+    table_name.replace(QRegExp("[ +*!#|]"), "_");
+    return table_name.toStdString();
 }
 
-void OSConfigurator_linux24::printChecksForRunTimeMultiAddress()
+void OSConfigurator_linux24::registerMultiAddressObject(MultiAddressRunTime *at)
 {
-    output << "# Using " << address_table_objects.size() << " address table files" << endl;
-
-    map<string,string>::iterator i;
-    for (i=address_table_objects.begin(); i!=address_table_objects.end(); ++i)
-    {
-        string at_name = i->first;
-        string at_file = i->second;
-        output << "check_file \"" + at_name + "\" \"" + at_file + "\"" << endl;
-    }
+    // std::map<std::string,std::string>
+    address_table_objects[normalizeSetName(at->getName())] = at->getSourceName();
 }
 
 int  OSConfigurator_linux24::prolog()
@@ -311,6 +322,15 @@ string OSConfigurator_linux24::printShellFunctions(bool have_ipv6)
     configlet.removeComments();
     configlet.collapseEmptyStrings(true);
 
+    configlet.setVariable("load_modules", options->getBool("load_modules"));
+
+    if (options->getBool("load_modules") ||
+        options->getBool("configure_vlan_interfaces") ||
+        options->getBool("configure_bonding_interfaces"))
+    {
+        configlet.setVariable("need_modprobe", true);
+    }
+
     if (options->getBool("verify_interfaces") || 
         options->getBool("manage_virtual_addr") ||
         options->getBool("configure_interfaces") )
@@ -323,6 +343,8 @@ string OSConfigurator_linux24::printShellFunctions(bool have_ipv6)
                               options->getBool("configure_bonding_interfaces"));
     }
 
+    configlet.setVariable("need_ipset", using_ipset);
+
     configlet.setVariable("need_iptables_restore",
                           options->getBool("use_iptables_restore"));
 
@@ -362,6 +384,37 @@ string OSConfigurator_linux24::printShellFunctions(bool have_ipv6)
     return output.join("\n").toStdString();
 }
 
+string OSConfigurator_linux24::printRunTimeAddressTablesCode()
+{
+    Configlet conf(fw, "linux24", "run_time_address_tables");
+    conf.setVariable("using_ipset", using_ipset);
+
+    ostringstream check_ostr;
+    ostringstream load_ostr;
+    map<string,string>::iterator i;
+    for (i=address_table_objects.begin(); i!=address_table_objects.end(); ++i)
+    {
+        string at_name = i->first;
+        string at_file = i->second;
+        // If the file name is empty, this run-time table is
+        // completely controlled by the user outside fwbuilder so we
+        // do not need to add commands to check if the file exits and
+        // load it
+        if (!at_file.empty())
+        {
+            check_ostr << "check_file \"" + at_name +
+                "\" \"" + at_file + "\"" << endl;
+            load_ostr << "reload_address_table \"" + at_name +
+                "\" \"" + at_file + "\"" << endl;
+        }
+    }
+
+    conf.setVariable("check_files_commands", check_ostr.str().c_str());
+    conf.setVariable("load_files_commands", load_ostr.str().c_str());
+
+    return conf.expand().toStdString();
+}
+
 string OSConfigurator_linux24::getPathForATool(const std::string &os_variant, OSData::tools tool_name)
 {
     FWOptions* options = fw->getOptionsObject();
@@ -468,7 +521,12 @@ string OSConfigurator_linux24::printRunTimeWrappers(FWObject *rule,
  */
 
     bool wildcard_interface = false;
-    QString combined_command = addressTableWrapper(rule, command.c_str(), ipv6);
+    QString combined_command;
+
+    if (using_ipset)
+        combined_command = command.c_str();
+    else
+        combined_command = addressTableWrapper(rule, command.c_str(), ipv6);
 
     command_wrappers->clear();
     command_wrappers->removeComments();
@@ -477,7 +535,7 @@ string OSConfigurator_linux24::printRunTimeWrappers(FWObject *rule,
 
     command_wrappers->setVariable("address_table", false);
 
-    QRegExp intf_re("\\$i_(\\S+)");
+    QRegExp intf_re("\\$i_([^ :]+)");
 
     QStringList iface_names;
     QStringList iface_vars;
@@ -485,13 +543,19 @@ string OSConfigurator_linux24::printRunTimeWrappers(FWObject *rule,
     while ((pos = intf_re.indexIn(combined_command, pos + 1)) > -1)
     {
         QString name = intf_re.cap(1);
+        int match_len = intf_re.matchedLength();
         iface_names.push_back(name);
         iface_vars.push_back("$i_" + name);
         if (name.contains("*")) 
         {
             wildcard_interface = true;
-            QString intf_family = name.section('*', 0);
+            QString intf_family = name.section('*', 0, 0);
             command_wrappers->setVariable("interface_family_name", intf_family);
+
+            // replace $i_ppp* with $addr. This must match shell code
+            // in the configlet run_time_wrappers
+            combined_command.replace(pos, match_len, "$addr");
+
             break;
         }
     }
@@ -500,7 +564,8 @@ string OSConfigurator_linux24::printRunTimeWrappers(FWObject *rule,
 
     if (!no_wrapper)
     {
-        QStringList command_lines = QString(combined_command).split("\n", QString::SkipEmptyParts);
+        QStringList command_lines = 
+            QString(combined_command).split("\n", QString::SkipEmptyParts);
         if (command_lines.size() > 1)
         {
             command_lines.push_front("{");
@@ -520,104 +585,13 @@ string OSConfigurator_linux24::printRunTimeWrappers(FWObject *rule,
     {
         QString intf_name = iface_names[idx];
         //if (ipv6) intf_name += "_v6";
-        command_wrappers->setVariable(QString("intf_%1_var_name").arg(idx+1), intf_name);
+        command_wrappers->setVariable(QString("intf_%1_var_name").arg(idx+1),
+                                      intf_name);
     }
 
     command_wrappers->setVariable("command", combined_command);
 
     return command_wrappers->expand().toStdString() + "\n";
-
-#if 0
-    string command_line = command;
-    ostringstream  ext_command_line;
-
-    int nlines = 0;
-    string::size_type p1 = 0;
-    string::size_type p2, p3;
-
-    p1=command_line.find("$at_");
-    if ( p1!=string::npos )
-    {
-        p2=command_line.find(" ",p1);
-        string at_var= command_line.substr(p1+1,p2-p1-1);  // skip '$'
-        string atfile = rule->getStr("address_table_file");
-        ext_command_line << "grep -Ev '^#|^;|^\\s*$' " << atfile << " | ";
-        ext_command_line << "while read L ; do" << endl;
-        ext_command_line << "  set $L; " << at_var << "=$1; ";
-        ext_command_line << command_line;
-        ext_command_line << "done" << endl;
-
-        command_line = ext_command_line.str();
-    }
-
-    p1 = 0;
-    while (1)
-    {
-        p1=command_line.find_first_of("\n\r",p1);
-        if (p1==string::npos) break;
-        nlines++;
-        p1=command_line.find_first_not_of("\n\r",p1);
-        if (p1==string::npos) break;
-    }
-
-    string getaddr_function_name = "getaddr";
-    if (ipv6) getaddr_function_name = "getaddr6";
-
-    ostringstream  res;
-    bool wildcard_interfaces = false;
-    p1=0;
-    while ((p1=command_line.find("$i_", p1))!=string::npos)
-    {
-        string  iface_name;
-        string  iface_var;
-
-        p2=command_line.find(" ",p1);
-        p3=command_line.find("_",p1) +1;
-        iface_name=command_line.substr(p3,p2-p3);
-        iface_var= command_line.substr(p1,p2-p1);
-
-/* if interface name ends with '*', this is a wildcard interface. */
-        string::size_type p4;
-        if ((p4=iface_name.find("*"))!=string::npos)
-        {
-            wildcard_interfaces = true;
-            string cmdline=command_line;
-            string iface_family_name=iface_name.substr(0,p4);
-            res << "getinterfaces " << iface_family_name << " | while read I; do" << endl;
-            res << "  ivar=`getInterfaceVarName $I`" << endl;
-            res << "  " << getaddr_function_name << " $I $ivar" << endl;
-            res << "  cmd=\"$\"$ivar"   << endl;
-            res << "  eval \"addr=$cmd\""          << endl;
-            cmdline.replace(p1,p2-p1,"$addr");
-            res << "  test -n \"$addr\" && ";
-            if (nlines>1) res << "{" << endl;
-            res << cmdline;
-            if (nlines>1) res << "}" << endl;
-            res << "done" << endl;
-        } else
-        {
-            // bug #1851166: there could be two dynamic interfaces in
-            // the same rule. Just print "test" command here and continue
-            // in the "while" loop. We'll print actual commands when the loop
-            // ends.
-            res << "test -n \"" << iface_var << "\" && ";
-        }
-        p1++;  // p1 points at the previous "$i_" fragment
-    }
-
-
-    // for wildcard interfaces we only support one such interface
-    // per rule and we have already printed the actual command above.
-    if (!wildcard_interfaces)
-    {
-        if (nlines>1) res << "{" << endl;
-        res << command_line;
-        if (nlines>1) res << "}" << endl;
-    }
-
-    return res.str();
-#endif
-
 }
 
 string OSConfigurator_linux24::printIPForwardingCommands()
