@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@vk.crocodile.org
 
-  $Id: PolicyCompiler_PrintRule.cpp 2829 2010-04-23 18:22:08Z vadim $
+  $Id$
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -56,6 +56,7 @@
 #include "Configlet.h"
 
 #include <QStringList>
+#include <QRegExp>
 
 #include <iostream>
 #if __GNUC__ > 3 || \
@@ -96,13 +97,13 @@ string PolicyCompiler_ipt::PrintRule::_printSingleOptionWithNegation(
     ostringstream ostr;
     if (XMLTools::version_compare(version, "1.4.3")>=0)
     {
-        ostr  << _printSingleObjectNegation(rel);
+        ostr << _printSingleObjectNegation(rel);
         ostr << option << " ";
         ostr << arg << " ";
     } else
     {
         ostr << option << " ";
-        ostr  << _printSingleObjectNegation(rel);
+        ostr << _printSingleObjectNegation(rel);
         ostr << arg << " ";
     }
     return ostr.str();
@@ -872,14 +873,34 @@ string PolicyCompiler_ipt::PrintRule::_printIP(IPService *srv, PolicyRule *rule)
         if (!ipt_comp->ipv6)
         {
             str << " -m ipv4options ";
-            if  (srv->getBool("any_opt")) str << " --any-opt";
-            else
+
+            if (version.empty() || XMLTools::version_compare(version, "1.4.3")<0)
             {
-                if  (srv->getBool("lsrr")) str << " --lsrr";
-                if  (srv->getBool("ssrr")) str << " --ssrr";
-                if  (srv->getBool("rr")) str << " --rr";
-                if  (srv->getBool("ts")) str << " --ts";
-                if  (srv->getBool("rtralt")) str << " --ra";
+                // "old" ipv4options module
+                if  (srv->getBool("any_opt")) str << " --any-opt";
+                else
+                {
+                    if  (srv->getBool("lsrr")) str << " --lsrr";
+                    if  (srv->getBool("ssrr")) str << " --ssrr";
+                    if  (srv->getBool("rr")) str << " --rr";
+                    if  (srv->getBool("ts")) str << " --ts";
+                    if  (srv->getBool("rtralt")) str << " --ra";
+                }
+            } else
+            {
+                // "new" ipv4options module
+                if  (srv->getBool("any_opt")) str << " --any";
+                else
+                {
+                    QStringList options;
+                    if  (srv->getBool("lsrr")) options << "lsrr";
+                    if  (srv->getBool("ssrr")) options << "ssrr";
+                    if  (srv->getBool("rr")) options << "record-route";
+                    if  (srv->getBool("ts")) options << "timestamp";
+                    if  (srv->getBool("rtralt")) options << "router-alert";
+                    if (options.size() > 0)
+                        str << " --flags " << options.join(",").toStdString();
+                }
             }
         } else
         {
@@ -1109,6 +1130,7 @@ string PolicyCompiler_ipt::PrintRule::_printDstService(RuleElementSrv  *rel)
 
 string PolicyCompiler_ipt::PrintRule::_printSrcAddr(RuleElement *rel, Address  *o)
 {
+    PolicyCompiler_ipt *ipt_comp=dynamic_cast<PolicyCompiler_ipt*>(compiler);
     string res;
     if (AddressRange::cast(o)!=NULL)
     {
@@ -1126,11 +1148,20 @@ string PolicyCompiler_ipt::PrintRule::_printSrcAddr(RuleElement *rel, Address  *
 
         return res;
     }
+
+    MultiAddressRunTime *atrt = MultiAddressRunTime::cast(o);
+    if (atrt!=NULL && atrt->getSubstitutionTypeName()==AddressTable::TYPENAME &&
+        ipt_comp->using_ipset)
+    {
+        return _printIpSetMatch(o, rel);
+    }
+
     return _printSingleOptionWithNegation(" -s", rel, _printAddr(o));
 }
 
 string PolicyCompiler_ipt::PrintRule::_printDstAddr(RuleElement *rel, Address  *o)
 {
+    PolicyCompiler_ipt *ipt_comp=dynamic_cast<PolicyCompiler_ipt*>(compiler);
     string res;
     if (AddressRange::cast(o)!=NULL)
     {
@@ -1147,9 +1178,38 @@ string PolicyCompiler_ipt::PrintRule::_printDstAddr(RuleElement *rel, Address  *
 
         return res;
     }
+
+    MultiAddressRunTime *atrt = MultiAddressRunTime::cast(o);
+    if (atrt!=NULL && atrt->getSubstitutionTypeName()==AddressTable::TYPENAME &&
+        ipt_comp->using_ipset)
+    {
+        return _printIpSetMatch(o, rel);
+    }
+
     return _printSingleOptionWithNegation(" -d", rel, _printAddr(o));
 }
 
+string PolicyCompiler_ipt::PrintRule::_printIpSetMatch(Address *o, RuleElement *rel)
+{
+    PolicyCompiler_ipt *ipt_comp=dynamic_cast<PolicyCompiler_ipt*>(compiler);
+    string set_name =
+        dynamic_cast<OSConfigurator_linux24*>(ipt_comp->osconfigurator)->normalizeSetName(o->getName());
+    string suffix = "dst";
+    if (RuleElementSrc::isA(rel)) suffix = "src";
+    if (RuleElementDst::isA(rel)) suffix = "dst";
+
+    string set_match_option;
+    if (XMLTools::version_compare(version, "1.4.4")>=0)
+        set_match_option = "--match-set";
+    else
+        set_match_option = "--set";
+
+    string set_match = set_match_option + " " + set_name + " " + suffix;
+    ostringstream ostr;
+    ostr << "-m set " << _printSingleOptionWithNegation("", rel, set_match);
+    return ostr.str();
+}
+
 string PolicyCompiler_ipt::PrintRule::_printAddr(Address  *o)
 {
     PolicyCompiler_ipt *ipt_comp=dynamic_cast<PolicyCompiler_ipt*>(compiler);
