@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@vk.crocodile.org
 
-  $Id: PolicyCompiler_ipt.h 2844 2010-04-26 16:27:57Z vadim $
+  $Id$
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -66,10 +66,13 @@ protected:
         bool                           have_dynamic_interfaces;
         bool                           have_connmark;
         bool                           have_connmark_in_output;
+        bool                           using_ipset;
+        bool                           actually_used_module_set;
         std::string                    my_table;
 
         std::map<std::string, int>     tmp_chain_no;
         std::map<std::string, int>     chain_usage_counter;
+        std::map<std::string, std::string> ipset_tables;
 
         typedef std::list<std::string> chain_list;
         std::map<std::string, chain_list*> chains;
@@ -159,6 +162,12 @@ protected:
          */
         DECLARE_POLICY_RULE_PROCESSOR(dropMangleTableRules);
 
+        /**
+         * this processor checks actions in mangle table. For example,
+         * REJECT is not allowed.
+         */
+        DECLARE_POLICY_RULE_PROCESSOR(checkActionInMangleTable);
+
         /**
          * adds few predefined (or "builtin") rules on top of the policy
          */
@@ -740,6 +749,19 @@ protected:
         };
         friend class PolicyCompiler_ipt::splitServicesIfRejectWithTCPReset;
 
+	/**
+	 * Rules that match icmpv6 should not be stateful. See SF bug 3094273
+         * Will reset "stateful" flag and issue warning.
+         * Call this processor after groups have been expanded in Srv
+	 */
+        class checkForStatefulICMP6Rules :public PolicyRuleProcessor
+        {
+            public:
+            checkForStatefulICMP6Rules(const std::string &name) : PolicyRuleProcessor(name) {}
+            virtual bool processNext();
+        };
+        friend class PolicyCompiler_ipt::checkForStatefulICMP6Rules;
+
 	/**
 	 * This processor separates TCP/UDP services with port ranges
 	 * (they can not be used with multiport). It also separates
@@ -1002,6 +1024,8 @@ public:
             virtual std::string _printDstAddr(libfwbuilder::RuleElement *rel,
                                               libfwbuilder::Address *o);
             virtual std::string _printAddr(libfwbuilder::Address *o);
+            virtual std::string _printIpSetMatch(
+                libfwbuilder::Address *o, libfwbuilder::RuleElement *rel);
             virtual std::string _printSingleObjectNegation(
                 libfwbuilder::RuleElement *rel);
             virtual std::string _printChain(libfwbuilder::PolicyRule *r);
