@@ -1,7 +1,7 @@
 #ifndef INC_IPTCfgParserTokenTypes_hpp_
 #define INC_IPTCfgParserTokenTypes_hpp_
 
-/* $ANTLR 2.7.7 (20090306): "iptables.g" -> "IPTCfgParserTokenTypes.hpp"$ */
+/* $ANTLR 2.7.7 (20100319): "iptables.g" -> "IPTCfgParserTokenTypes.hpp"$ */
 
 #ifndef CUSTOM_API
 # define CUSTOM_API
@@ -95,58 +95,63 @@ struct CUSTOM_API IPTCfgParserTokenTypes {
 		MATCH_RECENT_HITCOUNT = 84,
 		M_LENGTH = 85,
 		MATCH_LENGTH = 86,
-		M_MPORT = 87,
-		M_COMMENT = 88,
-		MATCH_COMMENT = 89,
-		MATCH_SRC_MULTIPORT = 90,
-		MATCH_DST_MULTIPORT = 91,
-		MATCH_BOTH_MULTIPORT = 92,
-		MATCH_ICMP_TYPE = 93,
-		MATCH_SRC_PORT = 94,
-		MATCH_SRC_PORT_SHORT = 95,
-		MATCH_DST_PORT = 96,
-		MATCH_DST_PORT_SHORT = 97,
-		MATCH_SYN = 98,
-		SYN = 99,
-		ACK = 100,
-		FIN = 101,
-		RST = 102,
-		URG = 103,
-		PSH = 104,
-		ALL = 105,
-		NONE = 106,
-		MATCH_TCP_FLAGS = 107,
-		MATCH_TCP_OPTION = 108,
-		Whitespace = 109,
-		IPV6 = 110,
-		MAC_ADDRESS = 111,
-		NEG_INT_CONST = 112,
-		HEXDIGIT = 113,
-		NUM_3DIGIT = 114,
-		NUM_HEX_4DIGIT = 115,
-		NUMBER = 116,
-		ULOG_QTHR = 117,
-		ULOG_NLG = 118,
-		ULOG_CPR = 119,
-		NUMBER_SIGN = 120,
-		PERCENT = 121,
-		AMPERSAND = 122,
-		APOSTROPHE = 123,
-		OPENING_PAREN = 124,
-		CLOSING_PAREN = 125,
-		PLUS = 126,
-		DOT = 127,
-		SEMICOLON = 128,
-		LESS_THAN = 129,
-		EQUALS = 130,
-		GREATER_THAN = 131,
-		QUESTION = 132,
-		COMMERCIAL_AT = 133,
-		CARET = 134,
-		UNDERLINE = 135,
-		OPENING_BRACE = 136,
-		CLOSING_BRACE = 137,
-		TILDE = 138,
+		M_PKTTYPE = 87,
+		MATCH_PKT_TYPE = 88,
+		WORD_BROADCAST = 89,
+		WORD_MULTICAST = 90,
+		WORD_UNICAST = 91,
+		M_MPORT = 92,
+		M_COMMENT = 93,
+		MATCH_COMMENT = 94,
+		MATCH_SRC_MULTIPORT = 95,
+		MATCH_DST_MULTIPORT = 96,
+		MATCH_BOTH_MULTIPORT = 97,
+		MATCH_ICMP_TYPE = 98,
+		MATCH_SRC_PORT = 99,
+		MATCH_SRC_PORT_SHORT = 100,
+		MATCH_DST_PORT = 101,
+		MATCH_DST_PORT_SHORT = 102,
+		MATCH_SYN = 103,
+		SYN = 104,
+		ACK = 105,
+		FIN = 106,
+		RST = 107,
+		URG = 108,
+		PSH = 109,
+		ALL = 110,
+		NONE = 111,
+		MATCH_TCP_FLAGS = 112,
+		MATCH_TCP_OPTION = 113,
+		Whitespace = 114,
+		IPV6 = 115,
+		MAC_ADDRESS = 116,
+		NEG_INT_CONST = 117,
+		HEXDIGIT = 118,
+		NUM_3DIGIT = 119,
+		NUM_HEX_4DIGIT = 120,
+		NUMBER = 121,
+		ULOG_QTHR = 122,
+		ULOG_NLG = 123,
+		ULOG_CPR = 124,
+		NUMBER_SIGN = 125,
+		PERCENT = 126,
+		AMPERSAND = 127,
+		APOSTROPHE = 128,
+		OPENING_PAREN = 129,
+		CLOSING_PAREN = 130,
+		PLUS = 131,
+		DOT = 132,
+		SEMICOLON = 133,
+		LESS_THAN = 134,
+		EQUALS = 135,
+		GREATER_THAN = 136,
+		QUESTION = 137,
+		COMMERCIAL_AT = 138,
+		CARET = 139,
+		UNDERLINE = 140,
+		OPENING_BRACE = 141,
+		CLOSING_BRACE = 142,
+		TILDE = 143,
 		NULL_TREE_LOOKAHEAD = 3
 	};
 #ifdef __cplusplus
