@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@vk.crocodile.org
 
-  $Id: CompilerDriver_ipf_run.cpp 3079 2010-07-14 17:31:28Z a2k $
+  $Id$
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -71,7 +71,7 @@
 #include <QFileInfo>
 #include <QFile>
 #include <QTextStream>
-
+#include <QtDebug>
 
 
 using namespace std;
@@ -104,6 +104,9 @@ QString CompilerDriver_ipf::composeActivationCommand(libfwbuilder::Firewall *fw,
 QString CompilerDriver_ipf::assembleManifest(Cluster*, Firewall* fw, bool )
 {
     FWOptions* options = fw->getOptionsObject();
+
+    //qDebug() << "CompilerDriver_ipf::assembleManifest fw_file_name=" << fw_file_name;
+
     QFileInfo fw_file_info(fw_file_name);
     QString ipf_file_name = fw_file_info.completeBaseName() + "-ipf.conf";
     QString nat_file_name = fw_file_info.completeBaseName() + "-nat.conf";
@@ -200,10 +203,14 @@ QString CompilerDriver_ipf::run(const std::string &cluster_id,
         }
 
         QString remote_ipf_name = options->getStr("ipf_conf_file_name_on_firewall").c_str();
-        if (remote_ipf_name.isEmpty()) remote_ipf_name = ipf_file_name;
+        if (remote_ipf_name.isEmpty())
+            remote_ipf_name = QFileInfo(ipf_file_name).fileName();
+        remote_ipf_name = this->escapeFileName(remote_ipf_name);
 
         QString remote_nat_name = options->getStr("nat_conf_file_name_on_firewall").c_str();
-        if (remote_nat_name.isEmpty()) remote_nat_name = nat_file_name;
+        if (remote_nat_name.isEmpty())
+            remote_nat_name = QFileInfo(nat_file_name).fileName();
+        remote_nat_name = this->escapeFileName(remote_nat_name);
 
         string s;
 
@@ -319,17 +326,7 @@ QString CompilerDriver_ipf::run(const std::string &cluster_id,
 
         if (have_filter) 
         {
-            QFileInfo finfo(ipf_file_name);
-            if (finfo.isRelative())
-            {
-                // if ipf_file_name is relative, it is relative to the
-                // directory the program started in, which can be
-                // different from wdir and different from the current dir
-                // at this point because we do chdir to the directory
-                // defined by the -d command line option
-                QFileInfo new_finfo(start_current_dir, ipf_file_name);
-                ipf_file_name = new_finfo.absoluteFilePath();
-            }
+            ipf_file_name = getAbsOutputFileName(ipf_file_name);
 
             info("Output file name: " + ipf_file_name.toStdString());
             QFile ipf_file(ipf_file_name);
@@ -367,17 +364,7 @@ QString CompilerDriver_ipf::run(const std::string &cluster_id,
 
         if (have_nat) 
         {
-            QFileInfo finfo(nat_file_name);
-            if (finfo.isRelative())
-            {
-                // if nat_file_name is relative, it is relative to the
-                // directory the program started in, which can be
-                // different from wdir and different from the current dir
-                // at this point because we do chdir to the directory
-                // defined by the -d command line option
-                QFileInfo new_finfo(start_current_dir, nat_file_name);
-                nat_file_name = new_finfo.absoluteFilePath();
-            }
+            nat_file_name = getAbsOutputFileName(nat_file_name);
 
             info("Output file name: " + nat_file_name.toStdString());
             QFile nat_file(nat_file_name);
@@ -418,17 +405,7 @@ QString CompilerDriver_ipf::run(const std::string &cluster_id,
             cluster, fw, !cluster_id.empty(), oscnf.get());
 
 
-        QFileInfo fw_finfo(fw_file_name);
-        if (fw_finfo.isRelative())
-        {
-            // if fw_file_name is relative, it is relative to the
-            // directory the program started in, which can be
-            // different from wdir and different from the current dir
-            // at this point because we do chdir to the directory
-            // defined by the -d command line option
-            QFileInfo new_finfo(start_current_dir, fw_file_name);
-            fw_file_name = new_finfo.absoluteFilePath();
-        }
+        fw_file_name = getAbsOutputFileName(fw_file_name);
 
         info("Output file name: " + fw_file_name.toStdString());
         QFile fw_file(fw_file_name);
