@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@vk.crocodile.org
 
-  $Id: CompilerDriver_ipfw_run.cpp 3079 2010-07-14 17:31:28Z a2k $
+  $Id$
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -305,17 +305,7 @@ QString CompilerDriver_ipfw::run(const std::string &cluster_id,
         QString script_buffer = assembleFwScript(
             cluster, fw, !cluster_id.empty(), oscnf.get());
 
-        QFileInfo finfo(fw_file_name);
-        if (finfo.isRelative())
-        {
-            // if fw_file_name is relative, it is relative to the
-            // directory the program started in, which can be
-            // different from wdir and different from the current dir
-            // at this point because we do chdir to the directory
-            // defined by the -d command line option
-            QFileInfo new_finfo(start_current_dir, fw_file_name);
-            fw_file_name = new_finfo.absoluteFilePath();
-        }
+        fw_file_name = getAbsOutputFileName(fw_file_name);
 
         info("Output file name: " + fw_file_name.toStdString());
         QFile fw_file(fw_file_name);
