@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@vk.crocodile.org
 
-  $Id: CompilerDriver_pf_run.cpp 3079 2010-07-14 17:31:28Z a2k $
+  $Id$
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -111,26 +111,29 @@ QString CompilerDriver_pf::printActivationCommands(Firewall *fw)
     string pfctl_dbg = (debug)?"-v ":"";
 
     QStringList activation_commands;
-    string remote_file = remote_conf_files["__main__"];
-    if (remote_file.empty()) remote_file = conf_files["__main__"];
+    QString remote_file = remote_conf_files["__main__"].c_str();
+    if (remote_file.isEmpty()) remote_file = conf_files["__main__"].c_str();
     if (remote_file[0] != '/') remote_file = "${FWDIR}/" + remote_file;
+    remote_file = this->escapeFileName(remote_file);
 
     activation_commands.push_back(
         composeActivationCommand(
-            fw, pfctl_dbg, "", fw->getStr("version"), remote_file));
+            fw, pfctl_dbg, "", fw->getStr("version"), remote_file.toStdString()));
 
     for (map<string,string>::iterator i=conf_files.begin();
          i!=conf_files.end(); ++i)
     {
-        string remote_file = remote_conf_files[i->first];
-        if (remote_file.empty()) remote_file = i->second;
+        QString remote_file = remote_conf_files[i->first].c_str();
+        if (remote_file.isEmpty()) remote_file = i->second.c_str();
         if (remote_file[0] != '/') remote_file = "${FWDIR}/" + remote_file;
+        remote_file = this->escapeFileName(remote_file);
 
         if (i->first != "__main__")
             activation_commands.push_back(
                 composeActivationCommand(
-                    fw, pfctl_dbg, i->first, fw->getStr("version"), remote_file));
+                    fw, pfctl_dbg, i->first, fw->getStr("version"), remote_file.toStdString()));
     }
+
     return activation_commands.join("\n");
 }
 
@@ -544,17 +547,7 @@ QString CompilerDriver_pf::run(const std::string &cluster_id,
 
             if (ruleset_name.find("/*")!=string::npos) continue;
 
-            QFileInfo finfo(file_name);
-            if (finfo.isRelative())
-            {
-                // if file_name is relative, it is relative to the
-                // directory the program started in, which can be
-                // different from wdir and different from the current dir
-                // at this point because we do chdir to the directory
-                // defined by the -d command line option
-                QFileInfo new_finfo(start_current_dir, file_name);
-                file_name = new_finfo.absoluteFilePath();
-            }
+            file_name = getAbsOutputFileName(file_name);
 
             info("Output file name: " + file_name.toStdString());
             QFile pf_file(file_name);
@@ -596,17 +589,7 @@ QString CompilerDriver_pf::run(const std::string &cluster_id,
         table_factories.clear();
         generated_scripts.clear();
 
-        QFileInfo finfo(fw_file_name);
-        if (finfo.isRelative())
-        {
-            // if fw_file_name is relative, it is relative to the
-            // directory the program started in, which can be
-            // different from wdir and different from the current dir
-            // at this point because we do chdir to the directory
-            // defined by the -d command line option
-            QFileInfo new_finfo(start_current_dir, fw_file_name);
-            fw_file_name = new_finfo.absoluteFilePath();
-        }
+        fw_file_name = getAbsOutputFileName(fw_file_name);
 
         info("Output file name: " + fw_file_name.toStdString());
         QFile fw_file(fw_file_name);
