@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@vk.crocodile.org
 
-  $Id: PolicyCompiler_ipf_writers.cpp 2342 2010-01-13 21:21:07Z vadim $
+  $Id$
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -114,7 +114,7 @@ string PolicyCompiler_ipf::PrintRule::_printPort(int rs,int re,bool neg)
     return str.str();
 }
 
-void PolicyCompiler_ipf::PrintRule::_printDstService(RuleElementSrv  *rel)
+void PolicyCompiler_ipf::PrintRule::_printDstService(RuleElement  *rel)
 {
     FWObject *o=rel->front();
     if (o && FWReference::cast(o)!=NULL) o=FWReference::cast(o)->getPointer();
