@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@vk.crocodile.org
 
-  $Id: PolicyCompiler_pf.cpp 2783 2010-03-29 20:01:10Z vadim $
+  $Id$
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -93,7 +93,11 @@ int PolicyCompiler_pf::prolog()
     return  PolicyCompiler::prolog();
 }
 
-
+/*
+ * this is very much like
+ * Compiler::swapMultiAddressObjectsInRE::processNext() except it also
+ * registers the table using registerTable()
+ */
 bool PolicyCompiler_pf::swapAddressTableObjectsInRE::processNext()
 {
     PolicyCompiler_pf *pf_comp=dynamic_cast<PolicyCompiler_pf*>(compiler);
@@ -453,32 +457,32 @@ void PolicyCompiler_pf::addDefaultPolicyRule()
 //        mgmt_workstation->setAddress(getCachedFwOpt()->getStr("mgmt_addr"));
             dbcopy->add(mgmt_workstation,false);
 
-
-            r= dbcopy->createPolicyRule();
+            r = dbcopy->createPolicyRule();
             temp_ruleset->add(r);
             r->setAction(PolicyRule::Accept);
             r->setLogging(false);
             r->setDirection(PolicyRule::Inbound);
-            r->setPosition(9999);
+            r->setPosition(9998);
             r->setComment("   backup ssh access rule ");
             r->setHidden(true);
             r->setFallback(false);
             r->setLabel("backup ssh access rule");
             r->setBool("needs_established",true);  // supported in ipfw
 
-            RuleElement *src=r->getSrc();
+            RuleElement *src = r->getSrc();
             assert(src!=NULL);
             src->addRef(mgmt_workstation);
 
-            RuleElement *dst=r->getDst();
+            RuleElement *dst = r->getDst();
             assert(dst!=NULL);
             dst->addRef(fw);
 
-            RuleElement *srv=r->getSrv();
+            RuleElement *srv = r->getSrv();
             assert(srv!=NULL);
             srv->addRef(ssh);
 
             combined_ruleset->push_front(r);
+
         }
 
         insertCarpRule();
