@@ -6,7 +6,7 @@ for directory in $(find . -name unit_tests)
 do
    home=`pwd`
    cd $directory
-   qmake -spec $QMAKESPEC || exit 1
+   [ -z $QMAKESPEC ] && { qmake || exit 1; } || { qmake -spec $QMAKESPEC || exit 1; }
    $action || exit 1
    cd $home
-done
\ No newline at end of file
+done
